import aiohttp as __aiohttp
import fileuploader.AuthError as AuthError
import fileuploader.exceptions as exceptions

class UploadResponse:
    def __init__(self, response_json):
        self.file_url: str = response_json["file_url"]
        self.file_url_full: str = response_json["file_url_full"]
        self.key: str = response_json["key"]
        self.ext: str = response_json["ext"]
        self.user_filename: str = response_json["user_filename"]
        self.synced: bool = response_json["synced"]
        self.size: str = response_json["size"]
        self.craeted_at: float = response_json["craeted_at"]
        self.auth_error: AuthError.AuthError = (
            AuthError.AuthError(response_json["auth_error"])
            if "auth_error" in response_json and response_json["auth_error"]
            else None
        )

    async def delete(self) -> bool:
        """Deletes file"""
        async with __aiohttp.ClientSession("https://fu.andcool.ru") as session:
            async with session.get(
                f"/api/delete/{self.file_url}?key={self.key}"
            ) as response:
                if response.status == 200:
                    return True

                errors = {
                    400: exceptions.InvalidUniqueKey,
                    404: exceptions.FileNotFound,
                    429: exceptions.TooManyRequests,
                    500: exceptions.InternalServerError,
                    502: exceptions.APIDidntRespond,
                    522: exceptions.ServerDidntRespond,
                }
                raise errors.get(response.status, exceptions.UnhandledError(response.status))

    def __str__(self):
        return f"<UploadResponse file_url:'{self.file_url}' user_filename:'{self.user_filename}'>"



class AuthError:
    def __init__(self, auth_error: dict[str, int]):
        self.message = None
        self.error_id = None
        if auth_error:
            self.message = auth_error['message']
            self.error_id = auth_error['errorId']

    def __str__(self):
        return f"<AuthError message:'{self.message}' error_id:{self.error_id}>"



import aiohttp as aiohttp
import fileuploader.exceptions as exceptions
from typing import List, Dict
import fileuploader.group as Group

class User:
    def __init__(self):
        self.username = None
        self.accessToken = None

    def __str__(self):
        return f"<User username:'{self.username}'>"
    
    async def logout(self) -> bool:
        """Logs out from account"""
        if not self.accessToken:
            raise exceptions.NotAuthorized("No access token provided")
        
        async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
            async with session.post(f"/api/logout", headers={"Authorization": "Bearer " + self.accessToken}) as response:
                if response.status == 401 or response.status == 200:
                    self.accessToken = None
                return response.status == 401 or response.status == 200
                
    async def transfer(self, data: List[Dict[str, str]]) -> list:
        """Transfers local files to an account"""
        if not self.accessToken:
            raise exceptions.NotAuthorized("No access token provided")
        
        async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
            async with session.post(f"/api/transfer", headers={"Authorization": "Bearer " + self.accessToken},
                                    json={
                                        "data": data
                                    }) as response:
                if response.status != 200:
                    raise exceptions.UnhandledError(await response.json())
                return (await response.json())['unsuccess']
            
    async def get_groups(self) -> List[Group.Group]:
        if not self.accessToken:
            raise exceptions.NotAuthorized("No access token provided")
        
        async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
            async with session.get(f"/api/get_groups", headers={"Authorization": "Bearer " + self.accessToken}) as response:
                if response.status != 200:
                    raise exceptions.UnhandledError(await response.json())
                
                groups_list: List[Group.Group] = []
                for group in (await response.json())['groups']:
                    group_obj = Group.Group()
                    group_obj.group_name = group["name"]
                    group_obj.group_id = group["group_id"]
                    groups_list.append(group_obj)

                private_group = Group.Group()
                private_group.group_id = "private"
                private_group.group_name = "private"
                groups_list.insert(0, private_group)
                return groups_list
            
    async def create_group(self, group_name: str) -> Group.Group:
        """Create group"""
        if not self.accessToken:
            raise exceptions.NotAuthorized("No access token provided")
        
        async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
            async with session.post(f"/api/create_group", 
                                    json={"group_name": group_name},
                                    headers={"Authorization": "Bearer " + self.accessToken}) as response:
                if response.status == 200:
                    response_json = await response.json()
                    group = Group.Group()
                    group.group_id = response_json['group_id']
                    group.group_name = response_json['name']
                    return group

                errors = {
                    400: exceptions.Error((await response.json())['message']),
                }
                raise errors.get(response.status, exceptions.UnhandledError(response.status))


async def login(username: str, password: str, is_bot: bool = False) -> User:
    """Log in by username and password"""
    async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
        async with session.post(f"/api/login?bot={str(is_bot).lower()}", 
                                json={"username": username,
                                      "password": password}) as response:
            if response.status == 200:
                response_json = await response.json()
                user = User()
                user.username = response_json['username']
                user.accessToken = response_json['accessToken']
                return user

            errors = {
                400: exceptions.NoUsernamePasswordProvided,
                403: exceptions.WrongPassword,
                404: exceptions.UserNotFound,
                500: exceptions.InternalServerError,
                502: exceptions.APIDidntRespond,
                522: exceptions.ServerDidntRespond
            }
            raise errors.get(response.status, exceptions.UnhandledError(response.status))
        

async def register(username: str, password: str, is_bot: bool = False) -> User:
    """Register by username and password"""
    async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
        async with session.post(f"/api/register?bot={str(is_bot).lower()}", 
                                json={"username": username,
                                      "password": password}) as response:
            if response.status == 200:
                response_json = await response.json()
                user = User()
                user.username = response_json['username']
                user.accessToken = response_json['accessToken']
                return user

            errors = {
                400: exceptions.UserAreadyRegistered(),
                404: exceptions.UserNotFound,
                500: exceptions.InternalServerError,
                502: exceptions.APIDidntRespond,
                522: exceptions.ServerDidntRespond
            }
            raise errors.get(response.status, exceptions.UnhandledError(response.status))
        

async def refresh_token(token: str) -> str:
    """Refreshes an access token"""
    async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
        async with session.post(f"/api/refresh_token", 
                                json={"accessToken": token}) as response:
            if response.status == 200:
                response_json = await response.json()
                return response_json['accessToken']

            raise exceptions.UnhandledError(response.status)
        

async def loginToken(token: str) -> User:
    """Log in by username and password"""
    async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
        async with session.get(f"/api/login/token", 
                                headers={"Authorization": "Bearer " + token}) as response:
            if response.status == 200:
                response_json = await response.json()
                user = User()
                user.username = response_json['username']
                user.accessToken = response_json['accessToken']
                return user

            if response.status == 401:
                response_json = await response.json()
                raise exceptions.NotAuthorized(response_json['auth_error']['message'])
            
            raise exceptions.UnhandledError(response.status)



import aiohttp as aiohttp
import fileuploader.exceptions as exceptions
from . import UploadResponse
from typing import List


class Group:
    def __init__(self):
        self.group_id: str = None
        self.group_name: str = None

    def __str__(self):
        return f"<Group group_name:{self.group_name}, group_id:{self.group_id}>"
    
    async def delete(self, user) -> bool:
        """Delete group"""
        if not self.group_id or not user.accessToken:
            raise exceptions.NotAuthorized("The user or group has not been initialized")
        
        async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
            async with session.post(f"/api/delete_group/{self.group_id}") as response:
                if response.status == 200:
                    return True

                errors = {
                    403: exceptions.Forbidden,
                    404: exceptions.GroupNotFound
                }
                raise errors.get(response.status, exceptions.UnhandledError(response.status))
            
    async def leave(self, user) -> bool:
        """Leave group"""
        if not self.group_id or not user.accessToken:
            raise exceptions.NotAuthorized("The user or group has not been initialized")
        
        async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
            async with session.post(f"/api/leave/{self.group_id}", 
                                   headers={"Authorization": "Bearer " + user.accessToken}) as response:
                if response.status == 200:
                    return True

                errors = {
                    400: exceptions.YouAreNotInTheGroup,
                    401: exceptions.NotAuthorized(""),
                    404: exceptions.GroupNotFound,
                    422: exceptions.InvalidGroup
                }
                raise errors.get(response.status, exceptions.UnhandledError(response.status))
            
    async def get_files(self, user) -> List[UploadResponse.UploadResponse]:
        """Get files in group"""
        if not self.group_id or not user.accessToken:
            raise exceptions.NotAuthorized("The user or group has not been initialized")
        
        async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
            async with session.get(f"/api/get_files/{self.group_id}", 
                                   headers={"Authorization": "Bearer " + user.accessToken}) as response:
                if response.status == 200:
                    files: List[UploadResponse.UploadResponse] = []
                    for file in (await response.json())['data']:
                        files.append(UploadResponse.UploadResponse(file))
                    return files

                errors = {
                    400: exceptions.GroupNotFound,
                    403: exceptions.Forbidden,
                    404: exceptions.GroupNotFound
                }
                raise errors.get(response.status, exceptions.UnhandledError(response.status))
            
    async def generate_invite(self, user):
        """Generate invite in group"""
        if not self.group_id or not user.accessToken:
            raise exceptions.NotAuthorized("The user or group has not been initialized")
        
        async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
            async with session.get(f"/api/generate_invite/{self.group_id}", 
                                   headers={"Authorization": "Bearer " + user.accessToken}) as response:
                if response.status == 200:
                    return InviteLink((await response.json())['invite_link']) 

                errors = {
                    400: exceptions.GroupNotFound,
                    403: exceptions.Forbidden,
                    404: exceptions.GroupNotFound,
                    422: exceptions.InvalidGroup
                }
                raise errors.get(response.status, exceptions.UnhandledError(response.status))

        
class InviteLink:
    def __init__(self, link: str):
        self.link: str = link.split("/")[-1]
        self.link_full = link

    def __str__(self):
        return f"<InviteLink link:{self.link}>"
    
    async def join(self, user) -> Group:
        """Join to the group by invite link"""
        if not user.accessToken:
            raise exceptions.NotAuthorized("The user has not been initialized")
        
        async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
            async with session.post(f"/api/join/{self.link}", 
                                    headers={"Authorization": "Bearer " + user.accessToken}) as response:
                if response.status == 200:
                    response_json = await response.json()
                    group = Group()
                    group.group_id = response_json['group_id']
                    group.group_name = response_json['name']
                    return group

                errors = {
                    400: exceptions.AlreadyInGroup,
                    404: exceptions.InvalidLink
                }
                raise errors.get(response.status, exceptions.UnhandledError(response.status))
            
    async def info(self, user) -> Group:
        """Get info about group by link"""
        if not user.accessToken:
            raise exceptions.NotAuthorized("The user has not been initialized")
        
        async with aiohttp.ClientSession("https://fu.andcool.ru") as session:
            async with session.get(f"/api/invite_info/{self.link}", 
                                    headers={"Authorization": "Bearer " + user.accessToken}) as response:
                if response.status == 200:
                    response_json = await response.json()
                    group = Group()
                    group.group_id = response_json['group_id']
                    group.group_name = response_json['name']
                    return group

                errors = {
                    404: exceptions.InvalidLink
                }
                raise errors.get(response.status, exceptions.UnhandledError(response.status))




class FileNotFound(Exception):
    """File not found"""
    pass

class TooManyRequests(Exception):
    """Too many requests"""
    pass

class InternalServerError(Exception):
    """Internal server error"""
    pass

class APIDidntRespond(Exception):
    """API down"""
    pass

class ServerDidntRespond(Exception):
    """Server down"""
    pass

class InvalidGroup(Exception):
    """Invalid group id"""
    pass

class YouAreNotInTheGroup(Exception):
    """You are not in this group"""
    pass

class GroupNotFound(Exception):
    """Group not found"""
    pass

class FileSizeExceedsTheLimit(Exception):
    """File size exceeds the limit (100MiB)"""
    pass

class InvalidUniqueKey(Exception):
    """Invalid unique key"""
    pass

class UserNotFound(Exception):
    """User not found"""
    pass

class WrongPassword(Exception):
    """Wrong password"""
    pass

class NoUsernamePasswordProvided(Exception):
    """No username/password provided"""
    pass

class Forbidden(Exception):
    """Forbidden"""
    pass

class AlreadyInGroup(Exception):
    """You are already in the group"""
    pass

class InvalidLink(Exception):
    """Invite link not found"""
    pass

class NotAuthorized(Exception):
    """Invalid access token or token expired+-"""
    def __init__(self, auth_mess, message="Not Authorized: "):
        self.message = "Not Authorized: " + auth_mess
        super().__init__(self.message)

class UserAreadyRegistered(Exception):
    def __init__(self, message="An account with this name is already registered"):
        self.message = message
        super().__init__(self.message)

class UnhandledError(Exception):
    def __init__(self, error_code, message="Unhandled error was occured: Error code"):
        self.message = f"Unhandled error was occured: Error code {error_code}"
        super().__init__(self.message)

class Error(Exception):
    def __init__(self, mess, message=""):
        self.message = f"An error has occured: {mess}"
        super().__init__(self.message)


from setuptools import setup

setup(
    name='fileuploader',
    version='0.1.3',    
    description='Package for working with the API fu.andcool.ru',
    url='https://github.com/Andcool-Systems/FileUploader-python',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    author='AndcoolSystems',
    author_email='main@andcool.ru',    
    license='GPL-3.0 license',
    packages=['fileuploader'],
    install_requires=['aiohttp'],
)    



"""
file uploader

Package for working with the API fu.andcool.ru
"""

import aiohttp as __aiohttp
import fileuploader.UploadResponse as UploadResponse
import fileuploader.exceptions as exceptions
import fileuploader.user as User
import fileuploader.AuthError as AuthError
import fileuploader.group as Group

__version__ = "0.1.3"
__author__ = 'AndcoolSystems'


async def upload(bytes: bytes, filename: str, user: User.User = None, group: Group.Group = None) -> UploadResponse.UploadResponse:
    form_data = __aiohttp.FormData()
    form_data.add_field('file', bytes, filename=filename)

    headers = {}
    group_id = "private"
    if user and user.accessToken:
        headers = {"Authorization": "Bearer " + user.accessToken}
        if group and group.group_id:
            group_id = group.group_id

    async with __aiohttp.ClientSession("https://fu.andcool.ru") as session:
        async with session.post(f"/api/upload/{group_id}", 
                                data=form_data, headers=headers) as response:
            if response.status == 200:
                return UploadResponse.UploadResponse(await response.json())
            
            errors = {  # List of known errors
                400: exceptions.InvalidGroup,
                401: exceptions.NotAuthorized(await response.json()['auth_error']),
                403: exceptions.YouAreNotInTheGroup,
                404: exceptions.GroupNotFound,
                413: exceptions.FileSizeExceedsTheLimit,
                429: exceptions.TooManyRequests,
                500: exceptions.InternalServerError,
                502: exceptions.APIDidntRespond,
                522: exceptions.ServerDidntRespond
            }
            raise errors.get(response.status, exceptions.UnhandledError(response.status))
        

async def delete(file_url: str, key: str) -> bool:
    """Deletes file"""
    async with __aiohttp.ClientSession("https://fu.andcool.ru") as session:
        async with session.get(f"/api/delete/{file_url}?key={key}") as response:
            if response.status == 200:
                return True

            errors = {
                400: exceptions.InvalidUniqueKey,
                404: exceptions.FileNotFound,
                500: exceptions.InternalServerError,
                502: exceptions.APIDidntRespond,
                522: exceptions.ServerDidntRespond
            }
            raise errors.get(response.status, exceptions.UnhandledError(response.status))
            